﻿using HeuristicLab.Algorithms.PushGP.Interpreter;

namespace HeuristicLab.Algorithms.PushGP.Expressions.Exec
{
    public class ExecDoRangeExpression : Expression
    {
        public override bool IsCodeOp { get { return false; } }

        public override void Eval(IInterpreter interpreter)
        {
            // not enough arguments on stack
            if (interpreter.IntegerStack.Count < 2 ||
                interpreter.ExecStack.Count == 0)
                return;

            var destinationIndex = interpreter.IntegerStack.Pop();
            var currentIndex = interpreter.IntegerStack.Pop();
            var loopBody = interpreter.ExecStack.Pop();

            if (destinationIndex == currentIndex)
            {
                interpreter.IntegerStack.Push(currentIndex);
                interpreter.ExecStack.Push(loopBody);
            }
            else
            {
                var nextIndex = destinationIndex < currentIndex
                    ? currentIndex - 1
                    : currentIndex + 1;

                interpreter.IntegerStack.Push(currentIndex);
                interpreter.IntegerStack.Push(nextIndex);
                interpreter.IntegerStack.Push(destinationIndex);

                var expression = new ExecDoRangeExpression();
                interpreter.ExecStack.Push(loopBody);
                interpreter.ExecStack.Push(loopBody);
                interpreter.ExecStack.Push(expression);
            }
        }

        public override string ToString()
        {
            return Symbols.ExecDoXRange;
        }
    }
}
