﻿using System;
using System.Collections.Generic;
using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Algorithms.PushGP.Stack;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    public abstract class Expression
    {
        public abstract bool IsCodeOp { get; }
        public abstract void Eval(IInterpreter interpreter);

        public void PushResult<T>(IStack<T> stack, int count, Func<T[], T> templateFunc)
        {
            PushResult(stack, stack, count, templateFunc);
        }

        public void PushResult<T, R>(IStack<T> sourceStack, IStack<R> targetStack, int count, Func<T[], R> templateFunc)
        {
            // not enough arguments on stack
            if (sourceStack.Count < count)
                return;

            var values = sourceStack.Pop(count);
            var result = templateFunc(values);

            targetStack.Push(result);
        }

        public void Duplicate<T>(IStack<T> stack)
        {
            // not enough arguments on stack
            if (stack.Count == 0)
                return;

            var value = stack.Top;
            stack.Push(value);
        }

        public static void Define<T>(IStack<T> stack, IStack<string> nameStack, IDictionary<string, Expression> customExpressions, Func<T, Expression> creator)
        {
            // not enough arguments on stack
            if (nameStack.Count == 0 ||
                stack.Count == 0)
                return;

            var name = nameStack.Pop();
            var expression = creator(stack.Top);

            if (customExpressions.ContainsKey(name))
            {
                customExpressions[name] = expression;
            }
            else
            {
                customExpressions.Add(name, expression);
            }
        }
    }
}